#ifndef cOnchipFlashWriter_h
#define cOnchipFlashWriter_h


#include "NyLPC_stdlib.h"
typedef struct NyLPC_TcOnchipFlashWriter NyLPC_TcOnchipFlashWriter_t;

struct NyLPC_TcOnchipFlashWriter
{
	NyLPC_TUInt8 _work[256];
};

void NyLPC_cOnchipFlashWriter_initialize(NyLPC_TcOnchipFlashWriter_t* i_inst);

#define NyLPC_cOnchipFlashWriter_finalize(i)

NyLPC_TBool NyLPC_cOnchipFlashWriter_write(NyLPC_TcOnchipFlashWriter_t* i_inst,const void* i_dest,const void* i_src,NyLPC_TUInt32 i_size);
NyLPC_TBool NyLPC_cOnchipFlashWriter_writeSector(NyLPC_TcOnchipFlashWriter_t* i_inst,NyLPC_TUInt16 i_sector,NyLPC_TUInt32 i_offset,const void* i_src,NyLPC_TUInt32 i_size);
NyLPC_TBool NyLPC_cOnchipFlashWriter_elase(NyLPC_TcOnchipFlashWriter_t* i_inst,NyLPC_TUInt16 i_sector_s,NyLPC_TUInt16 i_sector_e);
NyLPC_TBool NyLPC_cOnchipFlashWriter_isOnchipFlash(const void* i_addr);


#endif
